from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class ModelProperty:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    subject: Optional[str] = None
    propertyTypes: Optional[List[str]] = None
    minOccurs: Optional[int] = None
    maxOccurs: Optional[int] = None
    hasType: Optional[List[str]] = None
    relationPropertyLabel: Optional[Dict[str, str]] = None
    sequenceNumber: Optional[int] = None
    belongsToModule: Optional[str] = None
    formsSymmetryWith: Optional[str] = None
    isAbstractionOf: Optional[str] = None
    refersTo: Optional[str] = None
    hasDataType: Optional[str] = None
    hasSimpleType: Optional[str] = None
    hasObjectType: Optional[str] = None
    hasValueFrom: Optional[str] = None
    hasSome: Optional[List[str]] = None
    hasMember: Optional[str] = None
    contains: Optional[str] = None
    hasSupplier: Optional[str] = None
    hasGeneralConcept: Optional[str] = None
    notification: Optional[Dict[str, str]] = None



from dataclasses import dataclass
from typing import Dict, Optional

from fdk_rdf_parser.classes import SkosCode, ThemeEU, ThemeLOS
from .reference_data_client import get_reference_data
from .utils import remove_trailing_slash


@dataclass
class DataServiceReferenceData:
    media_types: Optional[Dict[str, SkosCode]] = None


@dataclass
class DatasetReferenceData:
    provenancestatement: Optional[Dict[str, SkosCode]] = None
    rightsstatement: Optional[Dict[str, SkosCode]] = None
    frequency: Optional[Dict[str, SkosCode]] = None
    linguisticsystem: Optional[Dict[str, SkosCode]] = None
    referencetypes: Optional[Dict[str, SkosCode]] = None
    openlicenses: Optional[Dict[str, SkosCode]] = None
    location: Optional[Dict[str, SkosCode]] = None
    eu_themes: Optional[Dict[str, ThemeEU]] = None
    los_themes: Optional[Dict[str, ThemeLOS]] = None


@dataclass
class InformationModelReferenceData:
    linguisticsystem: Optional[Dict[str, SkosCode]] = None
    openlicenses: Optional[Dict[str, SkosCode]] = None
    location: Optional[Dict[str, SkosCode]] = None
    eu_themes: Optional[Dict[str, ThemeEU]] = None
    los_themes: Optional[Dict[str, ThemeLOS]] = None


def get_data_service_reference_data() -> DataServiceReferenceData:
    return DataServiceReferenceData(media_types=get_and_map_media_types())


def get_dataset_reference_data() -> DatasetReferenceData:
    return DatasetReferenceData(
        provenancestatement=get_and_map_reference_codes("provenancestatement"),
        rightsstatement=get_and_map_reference_codes("rightsstatement"),
        frequency=get_and_map_reference_codes("frequency"),
        linguisticsystem=get_and_map_reference_codes("linguisticsystem"),
        referencetypes=get_and_map_reference_codes("referencetypes"),
        openlicenses=get_and_map_open_licenses(),
        location=get_and_map_reference_codes("location"),
        eu_themes=get_and_map_themes_eu(),
        los_themes=get_and_map_themes_los(),
    )


def get_info_model_reference_data() -> InformationModelReferenceData:
    return InformationModelReferenceData(
        linguisticsystem=get_and_map_reference_codes("linguisticsystem"),
        openlicenses=get_and_map_open_licenses(),
        location=get_and_map_reference_codes("location"),
        eu_themes=get_and_map_themes_eu(),
        los_themes=get_and_map_themes_los(),
    )


def get_and_map_reference_codes(endpoint: str) -> Optional[Dict[str, SkosCode]]:
    codes = get_reference_data(f"/codes/{endpoint}")
    if codes is not None:
        return {
            remove_trailing_slash(str(code.get("uri"))): SkosCode(
                uri=str(code.get("uri")) if code.get("uri") is not None else None,
                code=str(code.get("code")) if code.get("code") is not None else None,
                prefLabel=code.get("prefLabel")
                if code.get("prefLabel") is not None
                else None,
            )
            for code in codes
        }
    else:
        return None


def get_and_map_themes_eu() -> Optional[Dict[str, ThemeEU]]:
    mapped = {}
    themes = get_reference_data("/themes")
    if themes is not None:
        for theme in themes:
            eu_theme = ThemeEU()
            eu_theme.add_values_from_dict(theme)
            mapped[str(theme.get("id"))] = eu_theme
    return mapped if len(mapped) > 0 else None


def get_and_map_themes_los() -> Optional[Dict[str, ThemeLOS]]:
    mapped = {}
    themes = get_reference_data("/los")
    if themes is not None:
        for theme in themes:
            los_theme = ThemeLOS()
            los_theme.add_values_from_dict(theme)
            mapped[str(theme.get("uri"))] = los_theme
    return mapped if len(mapped) > 0 else None


def get_and_map_media_types() -> Optional[Dict[str, SkosCode]]:
    media_types = {}
    codes = get_reference_data("/codes/mediatypes")
    if codes is not None:
        for code in codes:
            if code.get("code"):
                media_types[str(code.get("code"))] = SkosCode(
                    code=str(code.get("code")),
                    prefLabel={"nb": str(code.get("name"))}
                    if code.get("name")
                    else None,
                )
    return media_types if len(media_types) > 0 else None


def get_and_map_open_licenses() -> Optional[Dict[str, SkosCode]]:
    licenses_with_both_protocols = {}
    licenses = get_and_map_reference_codes("openlicenses")
    if licenses:
        for li in licenses:
            licenses_with_both_protocols[li] = licenses[li]
            if li.startswith("http://"):
                https_uri = "https://" + li.split("http://")[1]
                licenses_with_both_protocols[https_uri] = licenses[li]
            elif li.startswith("https://"):
                http_uri = "http://" + li.split("https://")[1]
                licenses_with_both_protocols[http_uri] = licenses[li]

    return (
        licenses_with_both_protocols if len(licenses_with_both_protocols) > 0 else None
    )



from .ns import (
    adms_uri,
    br_uri,
    dcat_ap_no_uri,
    dcat_api_uri,
    dcat_uri,
    dct_uri,
    dqv_iso_uri,
    dqv_uri,
    model_dcat_ap_no_uri,
    owl_time_uri,
    prov_uri,
    rov_uri,
    schema_uri,
    vcard_uri,
    xkos_uri,
)
from .utils import (
    catalog_ref,
    date_list,
    date_value,
    identifier_list,
    is_type,
    object_value,
    resource_list,
    value_list,
    value_translations,
)



from .extend_dataservice import extend_data_service_with_reference_data
from .extend_dataset import extend_dataset_with_reference_data
from .extend_info_model import extend_info_model_with_reference_data
from .reference_data import (
    DataServiceReferenceData,
    DatasetReferenceData,
    get_data_service_reference_data,
    get_dataset_reference_data,
    get_info_model_reference_data,
    InformationModelReferenceData,
)



from .catalog import parse_catalog
from .contactpoint import extract_contact_points
from .dataservice import parse_data_service
from .dataset import parse_dataset
from .dcat_resource import parse_dcat_resource
from .distribution import extract_distributions
from .harvest_meta_data import extract_meta_data
from .info_model import parse_information_model
from .publisher import extract_publisher
from .qualified_attribution import extract_qualified_attributions
from .quality_annotation import extract_quality_annotation
from .references import extract_references
from .skos_concept import extract_skos_concept
from .subject import extract_subjects
from .temporal import extract_temporal



from .catalog import Catalog
from .contactpoint import ContactPoint
from .data_distribution_service import DataDistributionService
from .dataservice import DataService
from .dataset import Dataset, PartialDataset
from .dcat_resource import PartialDcatResource
from .distribution import Distribution
from .harvest_meta_data import HarvestMetaData
from .info_model import InformationModel
from .publisher import Publisher
from .qualified_attribution import QualifiedAttribution
from .quality_annotation import QualityAnnotation
from .references import Reference
from .skos_code import SkosCode
from .skos_concept import SkosConcept
from .subject import Subject
from .temporal import Temporal
from .theme import ConceptSchema, ThemeEU, ThemeLOS



from dataclasses import dataclass
from typing import Dict, List, Optional

from .data_distribution_service import DataDistributionService
from .skos_concept import SkosConcept


@dataclass
class Distribution:
    uri: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    downloadURL: Optional[List[str]] = None
    accessURL: Optional[List[str]] = None
    license: Optional[List[SkosConcept]] = None
    openLicense: bool = False
    conformsTo: Optional[List[SkosConcept]] = None
    page: Optional[List[SkosConcept]] = None
    format: Optional[List[str]] = None
    type: Optional[str] = None
    accessService: Optional[List[DataDistributionService]] = None



from dataclasses import dataclass
from typing import Dict, Optional


@dataclass
class SkosCode:
    uri: Optional[str] = None
    code: Optional[str] = None
    prefLabel: Optional[Dict[str, str]] = None



import logging
from typing import Optional

import requests

from .utils import org_path_url, organization_url


def get_rdf_org_data(orgnr: Optional[str]) -> Optional[str]:

    try:
        response = requests.get(
            organization_url(orgnr), headers={"Accept": "text/turtle"},
        )
        response.raise_for_status()

        return response.text
    except requests.HTTPError as err:
        logging.error(f"Error response from organization-catalogue ({err})")

    return None


def get_org_path(org: str) -> Optional[str]:

    try:
        response = requests.get(org_path_url(org), headers={"Accept": "text/plain"},)
        response.raise_for_status()

        return response.text
    except requests.HTTPError as err:

        logging.error(f"Error response from organization-catalogue ({err})")

    return None



from dataclasses import dataclass
from typing import Optional


@dataclass
class ContactPoint:
    uri: Optional[str] = None
    fullname: Optional[str] = None
    email: Optional[str] = None
    organizationName: Optional[str] = None
    organizationUnit: Optional[str] = None
    hasURL: Optional[str] = None
    hasTelephone: Optional[str] = None



import os
import re
from typing import Optional

base_org_url = os.getenv(
    "ORGANIZATION_CATALOGUE_BASE_URI",
    "https://organization-catalogue.staging.fellesdatakatalog.digdir.no",
)


def organization_url(orgnr: Optional[str]) -> str:
    return (
        f"{base_org_url}/organizations/{orgnr}"
        if orgnr is not None
        else f"{base_org_url}/organizations"
    )


def org_path_url(org: str) -> str:
    return f"{base_org_url}/organizations/orgpath/{org}"


def organization_number_from_uri(uri: str) -> Optional[str]:
    match_uri_0 = "https://data.brreg.no/enhetsregisteret/api/enheter/"
    match_uri_1 = base_org_url + "/organizations/"
    match_str = "^(?:" + match_uri_0 + "|" + match_uri_1 + ")(\\d{9})$"
    match = re.compile(match_str).match(uri)
    return match.group(1) if match else None



from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS, FOAF

from fdk_rdf_parser.classes import PartialDataset
from fdk_rdf_parser.rdf_utils import (
    adms_uri,
    catalog_ref,
    dcat_ap_no_uri,
    dcat_uri,
    dqv_iso_uri,
    object_value,
    prov_uri,
    value_list,
    value_translations,
)
from .catalog import parse_catalog
from .dcat_resource import parse_dcat_resource
from .distribution import extract_distributions
from .harvest_meta_data import extract_meta_data
from .qualified_attribution import extract_qualified_attributions
from .quality_annotation import extract_quality_annotation
from .references import extract_references
from .skos_code import extract_skos_code, extract_skos_code_list
from .skos_concept import extract_skos_concept
from .subject import extract_subjects
from .temporal import extract_temporal


def parse_dataset(
    datasets_graph: Graph, record_uri: URIRef, dataset_uri: URIRef
) -> PartialDataset:
    quality_annotations = extract_quality_annotation(datasets_graph, dataset_uri)

    dataset = PartialDataset(
        id=object_value(datasets_graph, record_uri, DCTERMS.identifier),
        admsIdentifier=value_list(datasets_graph, dataset_uri, adms_uri("identifier")),
        harvest=extract_meta_data(datasets_graph, record_uri),
        accessRightsComment=value_list(
            datasets_graph, dataset_uri, dcat_ap_no_uri("accessRightsComment")
        ),
        distribution=extract_distributions(
            datasets_graph, dataset_uri, dcat_uri("distribution")
        ),
        sample=extract_distributions(datasets_graph, dataset_uri, adms_uri("sample")),
        spatial=extract_skos_code_list(datasets_graph, dataset_uri, DCTERMS.spatial),
        source=object_value(datasets_graph, dataset_uri, dcat_ap_no_uri("source")),
        objective=value_translations(
            datasets_graph, dataset_uri, dcat_ap_no_uri("objective")
        ),
        page=value_list(datasets_graph, dataset_uri, FOAF.page),
        temporal=extract_temporal(datasets_graph, dataset_uri),
        subject=extract_subjects(datasets_graph, dataset_uri),
        provenance=extract_skos_code(datasets_graph, dataset_uri, DCTERMS.provenance),
        accrualPeriodicity=extract_skos_code(
            datasets_graph, dataset_uri, DCTERMS.accrualPeriodicity
        ),
        hasAccuracyAnnotation=quality_annotations.get(
            dqv_iso_uri("Accuracy").toPython()
        ),
        hasCompletenessAnnotation=quality_annotations.get(
            dqv_iso_uri("Completeness").toPython()
        ),
        hasCurrentnessAnnotation=quality_annotations.get(
            dqv_iso_uri("Currentness").toPython()
        ),
        hasAvailabilityAnnotation=quality_annotations.get(
            dqv_iso_uri("Availability").toPython()
        ),
        hasRelevanceAnnotation=quality_annotations.get(
            dqv_iso_uri("Relevance").toPython()
        ),
        legalBasisForRestriction=extract_skos_concept(
            datasets_graph, dataset_uri, dcat_ap_no_uri("legalBasisForRestriction")
        ),
        legalBasisForProcessing=extract_skos_concept(
            datasets_graph, dataset_uri, dcat_ap_no_uri("legalBasisForProcessing")
        ),
        legalBasisForAccess=extract_skos_concept(
            datasets_graph, dataset_uri, dcat_ap_no_uri("legalBasisForAccess")
        ),
        conformsTo=extract_skos_concept(
            datasets_graph, dataset_uri, DCTERMS.conformsTo
        ),
        informationModel=extract_skos_concept(
            datasets_graph, dataset_uri, dcat_ap_no_uri("informationModel")
        ),
        references=extract_references(datasets_graph, dataset_uri),
        qualifiedAttributions=extract_qualified_attributions(
            datasets_graph, dataset_uri, prov_uri("qualifiedAttribution")
        ),
        catalog=parse_catalog(datasets_graph, record_uri),
    )

    dataset.add_values_from_dcat_resource(
        parse_dcat_resource(
            graph=datasets_graph,
            subject=dataset_uri,
            catalog_subject=catalog_ref(datasets_graph, record_uri),
        )
    )

    return dataset



from typing import List, Optional

from rdflib import Graph, URIRef

from fdk_rdf_parser.classes import Publisher, QualifiedAttribution
from fdk_rdf_parser.organizations import organization_number_from_uri
from fdk_rdf_parser.rdf_utils import (
    dcat_uri,
    object_value,
    prov_uri,
    resource_list,
)


def extract_qualified_attributions(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[QualifiedAttribution]]:
    values = []
    for resource in resource_list(graph, subject, predicate):
        publisher_uri = object_value(graph, resource, prov_uri("agent"))

        if isinstance(publisher_uri, str):
            publisher_id = organization_number_from_uri(publisher_uri)
            role = object_value(graph, resource, dcat_uri("hadRole"))

            values.append(
                QualifiedAttribution(
                    agent=Publisher(uri=publisher_uri, id=publisher_id), role=role
                )
            )

    return values if len(values) > 0 else None



from typing import Dict, Optional

from rdflib import Graph
from rdflib.namespace import FOAF, RDF

from .classes import (
    Catalog,
    DataService,
    Dataset,
    InformationModel,
    QualifiedAttribution,
)
from .organizations import get_rdf_org_data, publisher_from_fdk_org_catalog
from .parse_functions import parse_data_service, parse_dataset, parse_information_model
from .rdf_utils import dcat_uri, is_type, model_dcat_ap_no_uri
from .reference_data import (
    extend_data_service_with_reference_data,
    extend_dataset_with_reference_data,
    extend_info_model_with_reference_data,
    get_data_service_reference_data,
    get_dataset_reference_data,
    get_info_model_reference_data,
)


def parse_data_services(data_service_rdf: str) -> Dict[str, DataService]:
    data_services: Dict[str, DataService] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_data_service_reference_data()

    data_services_graph = Graph().parse(data=data_service_rdf, format="turtle")

    for record_uri in data_services_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = data_services_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            dcat_uri("DataService"), data_services_graph, primary_topic_uri
        ):
            data_service = parse_data_service(
                data_services_graph, record_uri, primary_topic_uri
            )

            data_service.publisher = (
                publisher_from_fdk_org_catalog(data_service.publisher, fdk_orgs)
                if data_service.publisher
                else None
            )

            data_service.catalog = extend_catalog_with_orgs_data(
                data_service.catalog, fdk_orgs
            )
            data_service = extend_data_service_with_reference_data(
                data_service, reference_data
            )

            data_services[primary_topic_uri.toPython()] = data_service

    return data_services


def parse_datasets(rdf_data: str) -> Dict[str, Dataset]:
    datasets_graph = Graph().parse(data=rdf_data, format="turtle")
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_dataset_reference_data()

    datasets: Dict[str, Dataset] = {}

    for record_uri in datasets_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = datasets_graph.value(record_uri, FOAF.primaryTopic)
        if primary_topic_uri is not None and is_type(
            dcat_uri("Dataset"), datasets_graph, primary_topic_uri
        ):
            partial_dataset = parse_dataset(
                datasets_graph, record_uri, primary_topic_uri
            )

            dataset = Dataset(
                publisher=publisher_from_fdk_org_catalog(
                    partial_dataset.publisher, fdk_orgs
                )
                if partial_dataset.publisher
                else None
            )

            dataset.add_values_from_partial(values=partial_dataset)

            dataset = extend_dataset_with_reference_data(dataset, reference_data)
            dataset = extend_dataset_with_orgs_data(dataset, fdk_orgs)
            dataset.catalog = extend_catalog_with_orgs_data(dataset.catalog, fdk_orgs)

            datasets[primary_topic_uri.toPython()] = dataset

    return datasets


def parse_information_models(info_models_rdf: str) -> Dict[str, InformationModel]:
    info_models: Dict[str, InformationModel] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_info_model_reference_data()

    info_models_graph = Graph().parse(data=info_models_rdf, format="turtle")

    for record_uri in info_models_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = info_models_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            model_dcat_ap_no_uri("InformationModel"),
            info_models_graph,
            primary_topic_uri,
        ):
            info_model = parse_information_model(
                info_models_graph, record_uri, primary_topic_uri
            )

            info_model.publisher = (
                publisher_from_fdk_org_catalog(info_model.publisher, fdk_orgs)
                if info_model.publisher
                else None
            )

            info_model.catalog = extend_catalog_with_orgs_data(
                info_model.catalog, fdk_orgs
            )
            info_model = extend_info_model_with_reference_data(
                info_model, reference_data
            )

            info_models[primary_topic_uri.toPython()] = info_model

    return info_models


def extend_catalog_with_orgs_data(
    catalog: Optional[Catalog], organizations_graph: Graph
) -> Optional[Catalog]:
    if catalog:
        catalog.publisher = publisher_from_fdk_org_catalog(
            catalog.publisher, organizations_graph
        )

    return catalog


def extend_dataset_with_orgs_data(
    dataset: Dataset, organizations_graph: Graph
) -> Dataset:
    if isinstance(dataset.qualifiedAttributions, list):
        dataset.qualifiedAttributions = list(
            map(
                lambda qa: enhance_qualified_attribution_agent(qa, organizations_graph),
                dataset.qualifiedAttributions,
            )
        )

    return dataset


def enhance_qualified_attribution_agent(
    qa: QualifiedAttribution, organizations_graph: Graph
) -> QualifiedAttribution:
    qa.agent = publisher_from_fdk_org_catalog(qa.agent, organizations_graph)

    return qa



from typing import List, Optional

from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS, FOAF

from fdk_rdf_parser.classes import Distribution
from fdk_rdf_parser.rdf_utils import (
    dcat_uri,
    dct_uri,
    object_value,
    resource_list,
    value_list,
    value_translations,
)
from .data_distribution_service import extract_data_distribution_services
from .skos_concept import extract_skos_concept


def extract_distributions(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[Distribution]]:
    values = []
    for resource in resource_list(graph, subject, predicate):
        resource_uri = None
        if isinstance(resource, URIRef):
            resource_uri = resource.toPython()

        values.append(
            Distribution(
                uri=resource_uri,
                title=value_translations(graph, resource, DCTERMS.title),
                description=value_translations(graph, resource, DCTERMS.description),
                downloadURL=value_list(graph, resource, dcat_uri("downloadURL")),
                accessURL=value_list(graph, resource, dcat_uri("accessURL")),
                license=extract_skos_concept(graph, resource, DCTERMS.license),
                conformsTo=extract_skos_concept(graph, resource, DCTERMS.conformsTo),
                page=extract_skos_concept(graph, resource, FOAF.page),
                format=value_list(graph, resource, dct_uri("format")),
                type=object_value(graph, resource, DCTERMS.type),
                accessService=extract_data_distribution_services(graph, resource),
            )
        )

    return values if len(values) > 0 else None



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fdk_rdf_parser',
 'fdk_rdf_parser.classes',
 'fdk_rdf_parser.organizations',
 'fdk_rdf_parser.parse_functions',
 'fdk_rdf_parser.rdf_utils',
 'fdk_rdf_parser.reference_data']

package_data = \
{'': ['*']}

install_requires = \
['isodate>=0.6.0,<0.7.0', 'rdflib>=5.0.0,<6.0.0', 'requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'fdk-rdf-parser',
    'version': '0.2.1',
    'description': '',
    'long_description': None,
    'author': 'NilsOveTen',
    'author_email': 'nils.ove.tendenes@digdir.no',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Dict, List, Optional

from fdk_rdf_parser.classes import Dataset, Distribution, SkosCode
from .reference_data import DatasetReferenceData
from .utils import (
    extend_eu_themes,
    extend_los_themes,
    extend_reference_types,
    extend_skos_code,
    extend_skos_code_list,
    remove_trailing_slash,
    split_los_from_eu_themes,
)


def extend_dataset_with_reference_data(
    dataset: Dataset, ref_data: DatasetReferenceData
) -> Dataset:
    dataset.accessRights = extend_skos_code(
        dataset.accessRights, ref_data.rightsstatement
    )
    dataset.provenance = extend_skos_code(
        dataset.provenance, ref_data.provenancestatement
    )
    dataset.accrualPeriodicity = extend_skos_code(
        dataset.accrualPeriodicity, ref_data.frequency
    )
    dataset.language = extend_skos_code_list(
        dataset.language, ref_data.linguisticsystem
    )
    dataset.spatial = extend_skos_code_list(dataset.spatial, ref_data.location)
    dataset.distribution = extend_distributions(
        dataset.distribution, ref_data.openlicenses
    )
    dataset.references = extend_reference_types(
        dataset.references, ref_data.referencetypes
    )

    split_themes = split_los_from_eu_themes(dataset.theme, ref_data.los_themes)
    dataset.losTheme = extend_los_themes(split_themes["los"], ref_data.los_themes)
    dataset.theme = extend_eu_themes(split_themes["eu"], ref_data.eu_themes)

    return dataset


def extend_distributions(
    distributions: Optional[List[Distribution]],
    open_licenses: Optional[Dict[str, SkosCode]],
) -> Optional[List[Distribution]]:
    if distributions is None or open_licenses is None:
        return distributions
    else:
        extended_distributions = []
        for dist in distributions:
            if dist.license is not None:
                extended_licenses = []
                for lic in dist.license:
                    ref_code = (
                        open_licenses.get(remove_trailing_slash(lic.uri))
                        if lic.uri is not None
                        else None
                    )
                    if ref_code is not None:
                        dist.openLicense = True
                        if lic.prefLabel is None:
                            lic.prefLabel = ref_code.prefLabel
                    extended_licenses.append(lic)
                dist.license = extended_licenses
            extended_distributions.append(dist)
        return extended_distributions



from dataclasses import dataclass
from typing import Any, Dict, List, Optional


@dataclass
class ConceptSchema:
    id: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    versioninfo: Optional[str] = None
    versionnumber: Optional[str] = None


@dataclass
class ThemeEU:
    id: Optional[str] = None
    code: Optional[str] = None
    startUse: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    conceptSchema: Optional[ConceptSchema] = None

    def add_values_from_dict(self: Any, dict: Dict) -> None:
        self.id = str(dict.get("id")) if dict.get("id") is not None else None
        self.code = str(dict.get("code")) if dict.get("code") is not None else None
        self.startUse = (
            str(dict.get("startUse")) if dict.get("startUse") is not None else None
        )
        self.title = dict.get("title")
        self.conceptSchema = (
            ConceptSchema(
                id=str(dict["conceptSchema"].get("id"))
                if dict["conceptSchema"].get("id") is not None
                else None,
                title=dict["conceptSchema"].get("title"),
                versioninfo=str(dict["conceptSchema"].get("versioninfo"))
                if dict["conceptSchema"].get("versioninfo") is not None
                else None,
                versionnumber=str(dict["conceptSchema"].get("versionnumber"))
                if dict["conceptSchema"].get("versionnumber") is not None
                else None,
            )
            if dict.get("conceptSchema") is not None
            else None
        )


@dataclass
class ThemeLOS:
    children: Optional[List[str]] = None
    parents: Optional[List[str]] = None
    isTema: Optional[bool] = None
    losPaths: Optional[List[str]] = None
    name: Optional[Dict[str, str]] = None
    definition: Optional[Dict[str, str]] = None
    uri: Optional[str] = None
    synonyms: Optional[List[str]] = None
    relatedTerms: Optional[List[str]] = None

    def add_values_from_dict(self: Any, dict: Dict) -> None:
        self.children = dict.get("children")
        self.parents = dict.get("parents")
        self.isTema = dict.get("isTema")
        self.losPaths = dict.get("losPaths")
        self.name = dict.get("name")
        self.definition = dict.get("definition")
        self.uri = str(dict.get("uri")) if dict.get("uri") is not None else None
        self.synonyms = dict.get("synonyms")
        self.relatedTerms = dict.get("relatedTerms")
